@@ -1,5 +1,3 @@
-from __future__ import absolute_import, print_function
-
 import distutils.version
 import os
 import re
@@ -29,26 +27,17 @@ except ImportError:
 stdout = getattr(sys.stdout, 'buffer', sys.stdout)
 stderr = getattr(sys.stderr, 'buffer', sys.stderr)
 
-is_not_python2 = sys.version_info[0] >= 3
-if is_not_python2:
 
-    def _sys2bytes(p):
-        if p is None:
-            return p
-        return p.encode('utf-8')
-
-    def _bytes2sys(p):
-        if p is None:
-            return p
-        return p.decode('utf-8')
-
-
-else:
-
-    def _sys2bytes(p):
+def _sys2bytes(p):
+    if p is None:
         return p
+    return p.encode('utf-8')
 
-    _bytes2sys = _sys2bytes
+
+def _bytes2sys(p):
+    if p is None:
+        return p
+    return p.decode('utf-8')
 
 
 def check(name, desc):
@@ -170,8 +159,6 @@ def has_baz():
 
 @check("bzr", "Breezy library and executable version >= 3.1")
 def has_bzr():
-    if not is_not_python2:
-        return False
     try:
         # Test the Breezy python lib
         import breezy
@@ -200,10 +187,26 @@ def has_rhg():
 
 
 @check("pyoxidizer", "running with pyoxidizer build as 'hg'")
-def has_rhg():
+def has_pyoxidizer():
     return 'PYOXIDIZED_INSTALLED_AS_HG' in os.environ
 
 
+@check(
+    "pyoxidizer-in-memory",
+    "running with pyoxidizer build as 'hg' with embedded resources",
+)
+def has_pyoxidizer_mem():
+    return 'PYOXIDIZED_IN_MEMORY_RSRC' in os.environ
+
+
+@check(
+    "pyoxidizer-in-filesystem",
+    "running with pyoxidizer build as 'hg' with external resources",
+)
+def has_pyoxidizer_fs():
+    return 'PYOXIDIZED_FILESYSTEM_RSRC' in os.environ
+
+
 @check("cvs", "cvs client/server")
 def has_cvs():
     re = br'Concurrent Versions System.*?server'
@@ -266,7 +269,10 @@ def has_executablebit():
 
 @check("suidbit", "setuid and setgid bit")
 def has_suidbit():
-    if getattr(os, "statvfs", None) is None or getattr(os, "ST_NOSUID") is None:
+    if (
+        getattr(os, "statvfs", None) is None
+        or getattr(os, "ST_NOSUID", None) is None
+    ):
         return False
     return bool(os.statvfs('.').f_flag & os.ST_NOSUID)
 
@@ -316,7 +322,7 @@ def has_cacheable_fs():
     fd, path = tempfile.mkstemp(dir='.', prefix=tempprefix)
     os.close(fd)
     try:
-        return util.cachestat(path).cacheable()
+        return util.cachestat(_sys2bytes(path)).cacheable()
     finally:
         os.remove(path)
 
@@ -420,12 +426,9 @@ def has_dulwich():
     "dulwich", "Dulwich >= %s", [
         '%d.%d.%d' % vers
         for vers in (
-            (0, 19, 10),
-            (0, 20, 0),
-            (0, 20, 3),
-            (0, 20, 4),
             (0, 20, 37),
             (0, 20, 44),
+            (0, 21, 0),
         )
     ]
 )
@@ -435,8 +438,21 @@ def has_dulwich_range(v):
     return dulwich.__version__ >= tuple(map(int, v.split('.')))
 
 
+@check("dulwich-rust", "Dulwich Python library using Rust bindings")
+def has_dulwich_rust():
+    try:
+        from dulwich import objects, __version__
+
+        return (
+            __version__ > (0, 22, 0)
+            and objects._parse_tree_py is not objects.parse_tree
+        )
+    except ImportError:
+        return False
+
+
 @check("pygit2", "pygit2 Python library")
-def has_git():
+def has_pygit2():
     try:
         import pygit2
 
@@ -640,7 +656,14 @@ def has_pyflakes():
 
 @check("pylint", "Pylint python linter")
 def has_pylint():
-    return matchoutput("pylint --help", br"Usage:[ ]+pylint", True)
+    try:
+        import pylint
+
+        pylint.__version__
+    except ImportError:
+        return False
+    else:
+        return True
 
 
 @check("clang-format", "clang-format C code formatter (>= 11)")
@@ -673,20 +696,22 @@ def has_pygments():
         return False
 
 
-@check("pygments25", "Pygments version >= 2.5")
-def pygments25():
+def getpygmentsversion():
     try:
         import pygments
 
         v = pygments.__version__
+
+        parts = v.split(".")
+        return (int(parts[0]), int(parts[1]))
     except ImportError:
-        return False
+        return (0, 0)
 
-    parts = v.split(".")
-    major = int(parts[0])
-    minor = int(parts[1])
 
-    return (major, minor) >= (2, 5)
+@checkvers("pygments", "Pygments version >= %s", (2.5, 2.11, 2.14))
+def has_pygments_range(v):
+    major, minor = v.split('.')[0:2]
+    return getpygmentsversion() >= (int(major), int(minor))
 
 
 @check("outer-repo", "outer repo")
@@ -762,7 +787,7 @@ def has_test_repo():
 
 
 @check("network-io", "whether tests are allowed to access 3rd party services")
-def has_test_repo():
+def has_network_io():
     t = os.environ.get("HGTESTS_ALLOW_NETIO")
     return t == "1"
 
@@ -887,9 +912,9 @@ def has_demandimport():
     return (not has_chg()) and os.environ.get('HGDEMANDIMPORT') != 'disable'
 
 
-# Add "py27", "py35", ... as possible feature checks. Note that there's no
+# Add "py36", "py37", ... as possible feature checks. Note that there's no
 # punctuation here.
-@checkvers("py", "Python >= %s", (2.7, 3.5, 3.6, 3.7, 3.8, 3.9))
+@checkvers("py", "Python >= %s", (3.6, 3.7, 3.8, 3.9, 3.10, 3.11))
 def has_python_range(v):
     major, minor = v.split('.')[0:2]
     py_major, py_minor = sys.version_info.major, sys.version_info.minor
@@ -907,7 +932,7 @@ def has_python3exe():
     py = 'python3'
     if os.name == 'nt':
         py = 'py -3'
-    return matchoutput('%s -V' % py, br'^Python 3.(5|6|7|8|9)')
+    return matchoutput('%s -V' % py, br'^Python 3.(6|7|8|9|10|11)')
 
 
 @check("pure", "running with pure Python code")
@@ -1089,8 +1114,8 @@ def has_repofncache():
 @check('dirstate-v2', 'using the v2 format of .hg/dirstate')
 def has_dirstate_v2():
     # Keep this logic in sync with `newreporequirements()` in `mercurial/localrepo.py`
-    return has_rust() and matchoutput(
-        'hg config format.exp-rc-dirstate-v2', b'(?i)1|yes|true|on|always'
+    return matchoutput(
+        'hg config format.use-dirstate-v2', b'(?i)1|yes|true|on|always'
     )
 
 
@@ -1149,11 +1174,11 @@ def has_pytype():
     return version and sv(_bytes2sys(version.group(0))) >= sv('2019.10.17')
 
 
-@check("rustfmt", "rustfmt tool at version nightly-2020-10-04")
+@check("rustfmt", "rustfmt tool at version nightly-2021-11-02")
 def has_rustfmt():
     # We use Nightly's rustfmt due to current unstable config options.
     return matchoutput(
-        '`rustup which --toolchain nightly-2020-10-04 rustfmt` --version',
+        '`rustup which --toolchain nightly-2021-11-02 rustfmt` --version',
         b'rustfmt',
     )
 
@@ -1177,3 +1202,17 @@ def has_lzma():
 @check("bash", "bash shell")
 def has_bash():
     return matchoutput("bash -c 'echo hi'", b'^hi$')
+
+
+@check("unicodefs", "Unicode-only file system")
+def has_unicode_filesystem():
+    try:
+        with tempfile.NamedTemporaryFile(
+            prefix="bøf".encode("latin-1"), dir=b"."
+        ):
+            return False
+    except Exception:
+        return True
+@check("bigendian", "big-endian CPU")
+def has_bigendian():
+    return sys.byteorder == 'big'
